<?php

// CHECK http//www.wonder-directory.com
// - Hitting /submit/ when logged out just displays a message - no login or redirec
// - Check its form params too (name of buttons?)

require_once( 'multi_curl.php' );
require_once( 'captchas.php' );

class DAB_Account {

  protected static $pen_name, $email, $base_username;

  protected static function load_settings() {
    $config = get_option( DAB_OPTIONS_KEY, array() );
    self::$pen_name = $config['pen_name'];
    self::$email = $config['email'];

    $username = strtolower( self::pen_name() );
    self::$base_username = preg_replace( '/\s+/', '', $username );
  }

  protected static function pen_name() {
    if ( !self::$pen_name ) self::load_settings();
    return self::$pen_name;
  }

  protected static function email() {
    if ( !self::$email ) self::load_settings();
    return self::$email;
  }

  protected static function base_username() {
    if ( !self::$base_username ) self::load_settings();
    return self::$base_username;
  }

  function __construct($root) {
    $this->root = preg_replace( '{/$}', '', $root);
    $this->username = self::base_username();
    $this->password = $this->generate_password();
    $this->email = self::email();
    $this->failed_logins = 0;
  }

  static private function _inflate( $row ) {
    $class = $row->type;
    $obj = new $class( $row->url );
    $obj->username = $row->username;
    $obj->password = $row->password;
    $obj->id = $row->id;
    $obj->errors = $row->notes;
    $obj->cookies = unserialize( $row->cookies );
    $obj->failed_logins = $row->failed_logins;
    $obj->working = $row->working;
    return $obj;
  }

  static function find( $id ) {
    global $wpdb;
    $row = $wpdb->get_row( $wpdb->prepare(
        "SELECT id,url,username,password,type,notes,cookies,failed_logins,working
           FROM {$wpdb->prefix}dab_accounts
          WHERE id=%d", $id ) );

    if ( $row ) {
      return self::_inflate( $row );
    }
  }

  static function load( $url ) {
    global $wpdb;
    $row = $wpdb->get_row(
        "SELECT id,url,username,password,type,notes,cookies,failed_logins,working
           FROM {$wpdb->prefix}dab_accounts
          WHERE url=\"$url\""
    );

    if ( $row ) {
      return self::_inflate( $row );
    }
  }

  static function num_accounts() {
    global $wpdb;
    return $wpdb->get_var( "SELECT count(*) FROM {$wpdb->prefix}dab_accounts" );
  }


  static function num_working_sites() {
    global $wpdb;
    return $wpdb->get_var( "SELECT count(*) FROM {$wpdb->prefix}dab_accounts WHERE working != 0 AND failed_logins < 25" );
  }

  // Return an array of the working sites.
  static function load_working_sites( ) {
    global $wpdb;
    $rows = $wpdb->get_results(
        "SELECT id,url,username,password,type,notes,cookies,failed_logins,working
           FROM {$wpdb->prefix}dab_accounts
          WHERE working != 0 AND failed_logins < 25"
          // ORDER BY url"
    );

    $out = array();
    foreach ( $rows as $row ) {
      $out[] = self::_inflate( $row );
    }
    return $out;

  }

  // Return an array of all sites, working or not.
  static function all_sites( $order = "working,failed_logins DESC" ) {
    global $wpdb;
    $rows = $wpdb->get_results(
        "SELECT id,url,username,password,type,notes,cookies,failed_logins,working
           FROM {$wpdb->prefix}dab_accounts
       ORDER BY $order"
    );

    $out = array();
    foreach ( $rows as $row ) {
      $out[] = self::_inflate( $row );
    }
    return $out;

  }


  protected function make_new_username() {
    $this->username = self::base_username() . mt_rand(100, 999);
  }

  protected function make_new_email() {
    # Can only do this on gmail addresses.
    if ( !preg_match( '/^(.*?)@((?:google|g)mail\.com)$/', self::email(), $matches ) )
      return false;

    list( , $local_part, $domain) = $matches;

    # Remove all current dots.
    $local_part = str_replace( '.', '', $local_part );

    $dot_after = mt_rand( 1, strlen($local_part)-1 );
    $this->email = preg_replace( "/^(.{" . $dot_after . "})/", "$1.", $local_part ) . "@$domain";
    dab_status( "{$this->root} Adjusting email to make it unique" );
    return true;
  }

  function save( $dubious = false ) {

    $notes = $this->errors === NULL
           ? NULL
           : ( is_string( $this->errors ) ? $this->errors : serialize( $this->errors ) );

    global $wpdb;
    if ( isset( $this->id ) ) {
      $res = $wpdb->update( "{$wpdb->prefix}dab_accounts", array (
        'username' => $this->username,
        'password' => $this->password,
        // Don't change this flag so the number never goes down
        // 'working' => ( $dubious ? 2 : ($this->errors ? 0 : 1) ),
        'notes' => $notes,
        'cookies' => serialize( $this->cookies ),
        'failed_logins' => $this->failed_logins
      ), array( 'id' => $this->id ) );
      if (!$res ) {
        $wpdb->print_error();
      }
    }
    else {
      $res = $wpdb->insert( "{$wpdb->prefix}dab_accounts", array (
        'url' => $this->root,
        'type' => get_class($this),
        'username' => $this->username,
        'password' => $this->password,
        'working' => ( $dubious ? 2 : ($this->errors ? 0 : 1) ),
        'notes' => $notes,
        'cookies' => serialize( $this->cookies )
      ) );
      if (!$res ) {
        $wpdb->print_error();
      }
      if ( $this->errors || $dubious ) {
        dab_debug("{$this->root} account failed");
      }
      else
        dab_status("{$this->root} account created: {$this->username}:{$this->password}");

      $this->id = $wpdb->insert_id;
    }
  }

  static $password_chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
  protected function generate_password($n = 12) {
    $string = "";

    for ( $l = 0; $l < $n; $l++ ) {
      $string .= self::$password_chars[ mt_rand( 0, strlen( self::$password_chars ) - 1 ) ];
    }
    return $string;
  }

  function on_captcha_get( $info, $data ) {
    dab_status( "{$this->root} Got catpcha image" );

    if ( $info['content_type'] == 'text/html' ) {
      $this->abort( $info, $data );
      return;
    }
    $this->cookies( $info[ 'cookies' ] );

    $solver = DAB_CaptchaService::instance()->solve( $data, $this );
  }

  function get_recaptcha( $info, $data ) {
    $this->captcha_type = 'recaptcha';
    # <iframe src="http://api.recaptcha.net/noscript?k=6LfR88QSAAAAACXFrMIxIBgptrprWx1YFWb5Qmo6" height="300" width="500" frameborder="0"></iframe>

    preg_match( '/<iframe.*?src="([^"]+recaptcha[^"]+)"[^>]*>/', $data, $matches );
    $this->recaptcha_url = $matches[1];
    dab_debug( "Getting recaptcha iframe {$this->recaptcha_url}" );
    dab_status( "{$this->root} Getting recaptcha" );

    $curl = dab_curl( array(
      CURLOPT_POST => false,
      CURLOPT_REFERER => $info[ 'url' ],
      CURLOPT_URL => $this->recaptcha_url
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_recaptcha_html' ),
      array( $this, 'abort' )
    );
  }

  function tag_to_assoc_array( $html ) {
    $attrs = array();

    preg_match_all( '/\b(\w+)="([^"]*)"/', $html, $pairs );

    foreach ( $pairs[1] as $i => $attr_name ) {
      $attrs[ $attr_name ] = $pairs[2][$i];
    }
    return $attrs;
  }

  function on_recaptcha_html( $info, $data ) {
    dab_debug( "{$this->root} extracting recaptcha fields and image" );

    # Need to extract out the hidden fields
    preg_match_all( '/<input.*?\s+type="hidden"\s+.*?>/', $data, $matches );

    $this->recaptcha_fields = array();
    foreach( $matches[0] as $html ) {
      $attrs = $this->tag_to_assoc_array( $html );
      $this->recaptcha_fields[ $attrs['name'] ] = $attrs['value'];
    }


    # Now get the image.
    preg_match( '/<img\s+.*?src="([^"]+).*?>/', $data, $matches );
    $img_url = $matches[1];

    # Work out if its /img?x=, http://foo/img?x= or img?x=
    if ( $img_url[0] == '/' ) {
      $url = preg_replace('{^(https?://.*?)/.*$}', "$1$img_url", $info[ 'url' ] );
    }
    elseif ( preg_match('{^https?://}', $img_url ) ) {
      $url = $img_url;
    }
    else {
      $url = preg_replace('{/[^/]*\?.*$}', "/$img_url", $info[ 'url' ] );
    }

    dab_debug( "{$this->root} recaptcha image is at $url" );

    $curl = dab_curl( array(
      CURLOPT_POST => false,
      CURLOPT_REFERER => $info[ 'url' ],
      CURLOPT_URL => $url
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_recaptcha_image' ),
      array( $this, 'abort' )
    );
  }

  function on_recaptcha_image( $info, $data ) {
    dab_status( "{$this->root} Got recatpcha image" );
    $solver = DAB_CaptchaService::instance()->solve( $data, array( $this, 'on_recaptcha_solved' ) );
  }

  function on_recaptcha_solved( $captcha, $task_id ) {
    $this->captcha_id = $task_id;

    dab_status( "{$this->root} ReCaptcha is $captcha" );

    # Now submit the captcha to get the value we put in the other on the original page
    $fields = $this->recaptcha_fields;
    unset( $this->recaptcha_fields );

    $fields[ 'recaptcha_response_field' ] = $captcha;

    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_REFERER => $this->recaptcha_url,
      CURLOPT_URL => $this->recaptcha_url,
      CURLOPT_POSTFIELDS => http_build_query( $fields )
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_recaptcha_post' ),
      array( $this, 'abort' )
    );

  }

  # Called after we've got the image and then submitted the sovled captcha.
  function on_recaptcha_post( $info, $data ) {
    # Pull out the captcha solution from inside the <textarea></textarea>

    if( !preg_match( '{<textarea[^>]*>(.*?)</textarea}', $data, $matches ) ) {
      DAB_CaptchaService::instance()->report_failure( $this->captcha_id );

      # Try again.
      if ( $this->attempt < 5 ) {
        $this->attempt++;
        dab_debug("Trying recaptcha again");
        $this->on_recaptcha_html( $info, $data );
      }
    }
    else {
      $this->on_captcha_solved( $matches[1], $this->captcha_id );
    }
  }

  function abort( $info, $output ) {
    dab_debug( "abort request for {$this->root} {$info['url']}" );
    $this->errors = print_r( $info, true ) . "\n----------------------------\n" . $output;
    $this->save();
  }

  function retry( $info, $data ) {
    if ($this->attempt <= 2) {
      if ( isset( $this->article ) )
        $this->start_submit_article();
      else
        $this->signup( $info, $data );
    }
  }

  function get_page_charset( $info, $data ) {
    if ( preg_match( "{<meta http-equiv=\"Content-Type\" content=\"text/html;\s*charset=(.*?)\"/>", $data, $matches ) )
      return $matches[1];
    if ( preg_match( "{text/html;\s*charset=(.*)}", $info['conetnt-type'], $matches ) )
      return $matches[1];
    return "ISO-8859-1";
  }

  function cookies( ) {
    if (func_num_args() == 1 && ( $c = func_get_arg( 0 ) ) ) {
      if ( !$this->cookies )
        $this->cookies = $c;
      else
        $this->cookies = array_merge( $this->cookies, $c );
    }
    return $this->cookies;
  }

  // Given the list of categories try to find a best match.
  // If we cant then look for 'Other', 'Misc' etc.
  // Falling back to just picking a random one.
  function category( $desired, $categories) {
    $keys = array_keys( $categories );

    if ($desired === NULL ) {
      # just pick a random one
      return $categories[ array_rand( $keys) ];
    }

    # "&" and "and" are equivalent
    $full_match = preg_replace( '/ (and|&) /', ' (?:and|&) ', preg_quote($desired) );

    # Split by ' -- ' - if the specific category can't be found, any one in the
    # parent category will do
    # "Foo -- Bar -- Baz" -> "Foo(?: -- Bar(?: -- Baz)?)?"
    $split = preg_split( '/ -- /', $desired );

    $desired = '';
    for( $i = count($split)-1; $i >= 0; $i-- ) {
      $part = preg_replace( '/ (and|&) /', ' (?:and|&) ', preg_quote( $split[ $i ] ) );
      if ($i > 0 )
        $desired = "(?: -- $part$desired)?";
      else
        $desired = "$part$desired";
    }

    $pattern = array(
      "/^$full_match$/",
      "/^$desired( -- .*|$)/",
      "/^(.*? -- )?$desired( -- .*|$)/",
      "/^(.*? -- )?$desired (and|&|&amp;) .*?( -- .*|$)/",
      "/^(.*? -- )?$desired, .*?( -- .*|$)/",
      "/^(.*? -- )?.*? (and|&|&amp;) $desired( -- .*|$)/",
      "/^(.*? -- )?.*?, $desired( -- .*|$)/",
      '/^(.*? -- )?(Misc(\.|ellaneous)?|Other|Uncategorized)( -- .*|$)/'
    );
    $priorities = array(
      "0", "1", "2", "3", "4", "5", "6", "7"
    );

    // preg_filter is php >=5.3 only
    $munged = preg_replace( $pattern, $priorities, $keys );
    $matches = array_diff( $munged, $keys );
    asort( $matches );
    # Matches will now be something like
    #  array( [419] => 1, [85] => 4, [82] => 4, [0] => 6 )
    # That is: index into $keys => $match priority
    #
    # First pop off anything of 'misc' priority, then examine what's left

    $misc = array();
    while( end( $matches ) == end($priorities) ) {
      $misc[] = $keys[ key( $matches ) ];
      array_pop($matches);
    }

    reset( $matches );
    if ( count($matches) && current($matches) == 0 ) {
      # We have an exact match - pick that
      $key_idx = key( $matches );
    }
    elseif ( count( $matches ) > 0) {
      # We have a matching category - pick one at random
      $key_idx = array_rand( $matches );
    }
    elseif ( count( $misc) ) {
      # No matching category, but we do have a 'Misc' like cat
      $key_idx = array_rand( $misc );
    }
    else {
      # Else just pick a random one!
      $key_idx = array_rand( $keys );
    }

    return $categories[ $keys[ $key_idx ] ];
  }

  function submit_article( $article ) {
    dab_status( "{$this->root} submitting article '{$article->title}'" );
    $this->article = $article;

    # Do it!
    $this->start_submit_article();
  }

  function login_failed() {
    $this->failed_logins++;
    $this->save();

    $blasted = @unserialize( get_post_meta( $this->article->post_id, DAB_META_KEY, true ) );
    # WTF how?
    if ( is_string( $blasted ) || $blasted === FALSE ) $blasted = array();
    $blasted[ $this->id ] = false;
    update_post_meta( $this->article->post_id, DAB_META_KEY, serialize( $blasted ) );
  }

  function login_succeded() {
    $this->failed_logins = 0;
    $this->errors = null;
    $this->save();
  }

  function on_captcha_failed( $err ) {
    dab_debug("{$this->root} Captcha failed: " . var_export( $err, true ) );
  }


  function record_submission_finished( $url, $info, $data, $matches = null ) {
    dab_status( "{$this->root} finished" );
    dab_debug( "record_submission_finished: $url, " . $info['url'] . ", " . !!$data );

    $blasted = @unserialize( get_post_meta( $this->article->post_id, DAB_META_KEY, true ) );

    # WTF how?
    if ( is_string( $blasted ) || $blasted === FALSE ) $blasted = array();

    $blasted[ $this->id ] = array(
      'info' => $info,
      // 'data' => $data,
      'alerts' => $matches
    );
    if ($data) $blasted[ $this->id ]['data'] = gzdeflate( $data, 9 );

    update_post_meta( $this->article->post_id, DAB_META_KEY, serialize( $blasted ) );

    if ( !$data ) {
      $count = get_post_meta( $this->article->post_id, DAB_ARTICLE_SUBMISSION_KEY, true );
      if ( !is_numeric( $count ) ) $count = 0;
      $count++;
      update_post_meta( $this->article->post_id, DAB_ARTICLE_SUBMISSION_KEY, $count );
    }

    if ( $url ) {
      add_post_meta( $this->article->post_id, DAB_ARTICLE_URLS_KEY, $url );
      $this->errors = null;
    }

    # Record the latest cookies
    $this->save();
  }

  function get_select_options( $name, $html ) {
  # <select name="category">...\n...</select>
    if ( !preg_match( "{<select[^>]+name=([\"']?)$name\\1[^>]*>(.*?)</select>}s", $html, $matches ) ) {
      dab_debug( "{$this->root} Error getting $name <select> from html" );
      return FALSE;
    }
    $out = array();

    preg_match_all('/<option[^>]+value=(["\']?)(.*?)\1[^>]*>(.*?)\s*(?=<\/option>|<option|$)/sD',  $matches[2], $matches);
    foreach( $matches[2] as $i => $val ) {
      $out[ $matches[3][$i] ] = $val;
    }
    return $out;
  }

  function best_select_option( $opts ) {
    for ($i = 1; $i < func_num_args(); ++$i ) {
      $opt = func_get_arg( $i );
      if ( isset( $opts[ $opt ] ) ) return $opts[ $opt ];
    }

    // Fall back to the first one (in the HTML)
    reset( $opts );
    return current( $opts );
  }

}










class DAB_ArticleSetup extends DAB_Account {

  function signup( ) {
    dab_status( "{$this->root} Getting signup page" );

    $curl = dab_curl( array(
      CURLOPT_REFERER => "{$this->root}/",
      CURLOPT_URL => "{$this->root}/login.php"
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_signup_get' ),
      array( $this, 'abort' )
    );
  }

  # Once we've hit login.php, then request the captcha
  function on_signup_get( $info, $output ) {

    $this->attempt++;

    $this->cookies( $info[ 'cookies' ] );

    if ( preg_match( '/securimage_show.php/', $output ) ) {
      dab_status( "{$this->root} Fetching captcha" );
      $curl = dab_curl( array(
        CURLOPT_POST => false,
        CURLOPT_REFERER => "{$this->root}/login.php",
        CURLOPT_URL => "{$this->root}/securimage/securimage_show.php",
        CURLOPT_COOKIE => $this->cookies()
      ) );

      DAB_MultiCurl::add(
        $curl,
        array( $this, 'on_captcha_get' ),
        array( $this, 'abort' )
      );
    }
    else {
      $this->do_signup_post();
    }
  }

  function on_captcha_solved( $captcha, $task_id ) {
    $this->captcha_id = $task_id;

    dab_status("{$this->root} Captcha is $captcha");

    $this->do_signup_post( $captcha );
  }

  function do_signup_post( $captcha = NULL ) {
    $params = array(
        'username' => $this->username,
        'fname' => self::pen_name(),
        'email' => $this->email,
        'pWord' => $this->password,
        'cpWord' => $this->password,
    );
    if ($captcha) $params['captcha_code'] = $captcha;

    $params = http_build_query( $params );
    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $params,
      CURLOPT_REFERER => "{$this->root}/login.php",
      CURLOPT_URL => "{$this->root}/doregister.php",
      CURLOPT_COOKIE => $this->cookies()
    ) );
    dab_debug( "{$this->root} Posting signup form $params" );
    dab_status( "{$this->root} Posting signup form" );

    DAB_MultiCurl::add( $curl, array( $this, 'on_signup_post' ), array( $this, 'abort' ) );

  }

  function on_signup_post( $info, $data ) {
    dab_debug( $info );
    dab_debug( $data );

    # "ERROR: incorrect verification code"
    $matches = NULL;
    if ( preg_match( '/(ERROR: incorrect verification code|Invalid Code Entered)/', $data, $matches ) > 0) {
      dab_debug( "{$this->root} captcha {$this->captcha_id} failed " . $matches[1] );
      DAB_CaptchaService::instance()->report_failure( $this->captcha_id );

      # Try again.
      return $this->retry( $info, $data );
    }
    elseif ( preg_match_all( '/<div\s+[^>]*class="alert"[^>]*>(.*?)</s', $data, $matches ) > 0) {
      dab_debug( "{$this->root} errored: " . print_r($matches[1], true ) );
      dab_debug( "{$this->root} html: " . print_r( $info, true ) );
      if ( $matches[1][0] == "That username address is already in use!" && count( $matches[1] ) == 1 ) {
        # Try again.
        if ( $this->attempt < 5 ) {
          dab_status( "{$this->root} Randomizing username" );
          $this->make_new_username();
          return $this->retry( $info, $data );
        }

      }
      elseif ( $matches[1][0] == "That email address is already in use!" && count( $matches[1] ) == 1 ) {
          if ( $this->make_new_email() ) {
            return $this->retry( $info, $data );
          }
      }
      elseif ( preg_match( "/Your Account has been Created/i", $matches[1][0] ) ) {
        $this->save();
        return;
      }

      $this->errors = $matches[1];

      dab_debug("{$this->root} failed for other reason: " . print_r($matches[1], true) );
      $this->save();
    }
    else {
      $this->save();
    }
  }


  function start_submit_article() {
    $curl = dab_curl( array(
      CURLOPT_URL => "{$this->root}/author/submit.php",
      CURLOPT_COOKIE => $this->cookies()
    ) );

    dab_status( "{$this->root} getting submission page" );
    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_submit_article_get' ),
      array( $this, 'abort' )
    );
  }

  function on_submit_article_get( $info, $data ) {
    if ( preg_match( '/login\.php/', $info['url'] ) ) {
      # Need to log in.
      return $this->login( $info, $data );
    }

    $cats = $this->get_categories( $info, $data );

    if (!$cats) {
      dab_debug( "{$this->root} Giving up - couldn't find categories" );
      return;
    }

    $cat_id = $this->category( $this->article->category, $cats );
    dab_status( "{$this->root} Category: " . array_search( $cat_id, $cats ) );

    $params = http_build_query( array(
      'body' => $this->article->content,
      'resource' => $this->article->bio,
      'title' => $this->article->title,
      'category' => $cat_id,
      'update' => ""
    ) );

    dab_debug( "{$this->root}: submit params " . print_r( $params, true ) );
    dab_status( "{$this->root} Posting submission page" );
    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $params,
      CURLOPT_REFERER => "{$this->root}/author/submit.php",
      CURLOPT_URL => $info['url'],
      CURLOPT_COOKIE => $this->cookies()
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_submit_article_post' ),
      array( $this, 'abort' )
    );
  }

  function on_submit_article_post( $info, $data ) {
    $this->cookies( $info[ 'cookies' ] );

    $url = NULL;
    $title = preg_quote( $this->article->title );
    # It might not be auto approve, so we need to check for the right edit link.
    if ( preg_match( '{author/articles\.php\?newarticle=true$}', $info['url'] ) &&
         preg_match_all( '{<tr>.*?'. $title . '.*?<a href="[^"]*?update.php\?id=(\d+)">\s*<b>Edit</b>\s*</a>.*?</tr>}s', $data, $edit_ids ) )
    {
      # Find the edit links to get the highest id.
      sort( $edit_ids[1], SORT_NUMERIC );

      $id = end( $edit_ids[1] );

      dab_debug("id is $id - checking for view link to it: \"/article.php?id={$id}/\"");

      # Check for a view link to see if its published already or not
      if ( strpos( $data, "article.php?id={$id}" ) !== false )
        $url = "{$this->root}/article.php?id=" . $id;
      $data = NULL;
    }

    $this->record_submission_finished( $url, $info, $data );
  }

  function login( $info, $data ) {
    # Make sure we don't loop
    if ( ++$this->login_attempt > 3 ) {
      $this->login_failed();
      return;
    }

    $this->cookies( $info[ 'cookies' ] );

    $params = http_build_query( array (
      'userid' => $this->username,
      'password' => $this->password
    ) );
    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $params,
      CURLOPT_REFERER => "{$this->root}/login.php",
      CURLOPT_URL => "{$this->root}/login.php",
      CURLOPT_COOKIE => $this->cookies()
    ) );


    dab_debug( "{$this->root} Posting login form $params" );
    dab_status( "{$this->root} logging in" );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_login_post' ),
      array( $this, 'abort' )
    );
  }

  function on_login_post( $info, $data ) {

    $this->cookies( $info['cookies'] );

    if ( preg_match( '/login\.php/', $info['url'] ) ) {
      if ( preg_match_all( '/<div\s+[^>]*class="alert"[^>]*>(.*?)</', $data, $matches ) > 0) {
        $this->notes .= "\n---\n" .print_r( $matches[1], true );
        $this->errors = $matches[1];
      }
      else {
        dab_debug( "{$this->root} Uhoh - stayed on login page!");
        dab_debug( $data );
      }
      $this->login_failed();
      return;
    }
    $this->login_succeded();

    $this->start_submit_article();
  }

  function get_categories( $info, $html ) {
    # <select name="category">...\n...</select>
    if ( !preg_match( '/<select name="category">(.*?)<\/select>/s', $html, $matches ) ) {
      dab_debug( "{$this->root} Error getting categories from html" );
      return;
    }

    $cats = array();

    # Arts & Entertaiment
    #  -- Celebrities
    $parent_cat;
    preg_match_all('{<option[^>]+value="([^"]+)"[^>]*>(.*?)</option>}',  $matches[1], $matches);
    foreach( $matches[1] as $i => $val ) {
      $cat = $matches[2][$i];
      if ( strpos( $cat, ' -- ' ) === FALSE ) {
        $parent_cat = $cat;
      }
      else {
        $cat = $parent_cat . $cat;
      }
      $cats[ $cat ] = $val;
    }

    return $cats;
  }

}












class DAB_ArticleMS extends DAB_Account {

  function __construct( $root ) {
    parent::__construct( $root );
    # First time through on ArticleMS try using the name with spaces
    $this->username = self::pen_name();
  }

  function signup( ) {
    dab_status( "{$this->root} Getting signup page" );

    if ( $this->root == 'http://legal-articles.deysot.com' )
      $url = "{$this->root}/registerantispam/";
    elseif ( $this->root == 'http://mortgagetips411.com' )
      $url = "{$this->root}/registernotreally/";
    else
      $url = "{$this->root}/register/";

    $curl = dab_curl( array(
      CURLOPT_REFERER => "{$this->root}/",
      CURLOPT_URL => $url
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_signup_get' ),
      array( $this, 'abort' )
    );
  }

  # Once we've hit /register/ see if there is a captcha or not (its an option on the site - nothing, captcha, recaptcha)
  function on_signup_get( $info, $data ) {
    $this->attempt++;
    $this->cookies( $info['cookies'] );

    # Deal with the Unique Article Wizard interstitial page
    if ( preg_match( "/preregister|preregistration/i", $info['url'] ) ||
         preg_match( "{<u>Unique Article Wizard</u>}", $data) ) {
      if (!$this->uaw_page_handled) {
        # Make sure we don't loop
        $this->uaw_page_handled = true;

        $curl = dab_curl( array(
          CURLOPT_POST => false,
          CURLOPT_REFERER => $info['url'],
          CURLOPT_URL => "{$this->root}/register/",
          CURLOPT_COOKIE => $this->cookies()
        ) );

        DAB_MultiCurl::add(
          $curl,
          array( $this, 'on_signup_get' ),
          array( $this, 'abort' )
        );
      }
    }
    elseif ( preg_match( '/<img\s+[^>]*src=".*?visual-captcha.php"[^>]*>/', $data ) ) {
      $this->get_builtin_captcha( $info, $data);
    }
    elseif( preg_match('{"http://api.recaptcha.net}', $data) ) {
      $this->get_recaptcha( $info, $data );
    }
    else {
      $this->do_signup_post( $info, $data );
    }

  }

  function get_builtin_captcha( $info, $data ) {
    dab_status("{$this->root} Fetching captcha");

    $this->captcha_type = 'builtin';

    $curl = dab_curl( array(
      CURLOPT_POST => false,
      CURLOPT_REFERER => "{$this->root}/register",
      CURLOPT_URL => "{$this->root}/visual-captcha.php",
      CURLOPT_COOKIE => $this->cookies()
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_captcha_get' ),
      array( $this, 'abort' )
    );
  }

  function on_captcha_solved( $captcha, $task_id ) {
    $this->captcha_id = $task_id;

    if ( $this->captcha_type != 'recaptcha' ) {
      dab_status("{$this->root} Captcha is $captcha");
    }

    $meth = $this->on_captcha_solved_method;
    if ( !$meth ) $meth = "do_signup_post";

    call_user_func( array( $this, $meth ), $captcha );
  }

  function do_signup_post( $captcha = NULL ) {
    $params = array(
      'user' => $this->username,
      'email' => $this->email,
      'passA' => $this->password,
      'passB' => $this->password,
      'submit' => 'Submit'
    );

    $this->set_captcha_fields( $params, $captcha );

    $params = http_build_query( $params );
    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $params,
      CURLOPT_REFERER => "{$this->root}/register/",
      CURLOPT_URL => "{$this->root}/register/",
      CURLOPT_COOKIE => $this->cookies()
    ) );
    dab_debug( "{$this->root} Posting signup form $params" );
    dab_status( "{$this->root} Posting signup form");
    DAB_MultiCurl::add( $curl, array( $this, 'on_signup_post' ), array( $this, 'abort' ) );

  }

  function on_signup_post( $info, $data ) {
    $matches = NULL;
    if ( preg_match( '/You must enter the verification code /', $data ) > 0) {
      dab_debug("captcha {$this->captcha_id} failed");
      DAB_CaptchaService::instance()->report_failure( $this->captcha_id );

      # Try again.
      $this->retry( $info, $data );
    }
    elseif ( preg_match_all( '/<div\s+style="color:red"[^>]*>\s*(.*?)\s*<\/div>/', $data, $matches ) > 0 ||
             preg_match_all('/<font color=red>\s*(.*?)\s*<\/font>/', $data, $matches) > 0 ) {

      foreach ($matches[1] as $err ) {
        if ( preg_match( "/An? user with that name already exists/", $err ) )
          $new_username = true;
        elseif( preg_match( "/An? user with that email already exists/i", $err ) ||
                $err == "An user with that Email allready registered" ) {
          $new_email = $this->make_new_email();
        }
      }

      if ($new_username || $new_email ) {
        if ( $this->attempt < 5 ) {
          dab_status( "{$this->root} Randomizing username" );
          $this->make_new_username();
        }
        return $this->retry( $info, $data );
      }

      $this->errors = $matches[1];
      dab_debug("{$this->root} failed for other reason: " . print_r($matches[1],true) );
      $this->save();
    }
    elseif (preg_match( "/ERROR: Duplicate entry '.*?' for key 'user_name'/", $data ) ) {
      # Lol special.
      if ( $this->attempt < 5 ) {
        dab_status( "{$this->root} Randomizing username" );
        $this->make_new_username();
        return $this->retry( $info, $data );
      }
    }
    elseif( preg_match( '/(You have successfully registered|Registrazione avvenuta con successo)/', $data ) ) {
      dab_debug( "{$this->root} huge success" );
      $this->save();
    }
    else {
      dab_debug( "Dubious result for {$this->root}" );
      $this->errors = $data;
      $this->save( true );
    }
  }


  function start_submit_article() {
    # Check if we are logged in first before we submit.
    if ( $this->root == "http://www.articlesisland.com" )
      $url = "{$this->root}/myaccount/";
    else
      $url = "{$this->root}/usercp/";

    $curl = dab_curl( array(
      CURLOPT_URL => $url,
      CURLOPT_COOKIE => $this->cookies()
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_get_usercp' ),
      array( $this, 'abort' )
    );
  }

  function on_get_usercp( $info, $data ) {
    $this->cookies( $info['cookies'] );
    if ( strpos( $data, 'You must be logged in to view this page' ) ) {
      $curl = dab_curl( array(
        CURLOPT_URL => "{$this->root}/login/",
        CURLOPT_REFERER => $info[ 'url' ],
        CURLOPT_COOKIE => $this->cookies()
      ) );

      DAB_MultiCurl::add(
        $curl,
        array( $this, 'login' ),
        array( $this, 'abort' )
      );
      return;
    }
    $this->get_submit_article_page( $info );
  }

  function get_submit_article_page( $info ) {
    $curl = dab_curl( array(
      CURLOPT_URL => "{$this->root}/submit/",
      CURLOPT_REFERER => $info[ 'url' ],
      CURLOPT_COOKIE => $this->cookies()
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_submit_article_get' ),
      array( $this, 'abort' )
    );
  }

  function on_submit_article_get( $info, $data ) {
    $cats = $this->get_categories( $info, $data );
    if (!$cats) {
      dab_debug( "{$this->root} Giving up - couldn't find categories" );
      dab_debug( $data );
      return;
    }

    $cat_id = $this->category( $this->article->category, $cats );
    dab_status( "{$this->root} Category: " . array_search( $cat_id, $cats ) );

    $params =  array(
      'text' => $this->article->content,
      'about' => $this->article->bio,
      'title' => $this->article->title,
      'category' => $cat_id,
      'keywords' => "",
      'tags' => "",
      'publish' => "Publish",
    );

    # Optional fields - only appear on some sites
    if ( $opts = $this->get_select_options( "article_state", $data ) ) {
      $params[ 'article_state' ] = $this->best_select_option( $opts, 'Published', 'Pending Review', 'Draft' );
    }

    if ( $opts = $this->get_select_options( "penname", $data ) ) {
      $params[ 'penname' ] = current( $opts );
    }

    if ( $opts = $this->get_select_options( "allow_comments", $data ) ) {
      $params[ 'allow_comments' ] = $opts[ 'Yes' ];
    }

    if ( $opts = $this->get_select_options( "collaboration_model", $data ) ) {
      $params[ 'collaboration_model' ] = $this->best_select_option( $opts, 'Closed Collaboration', 'Moderated Collaboration' );
    }

    # Some versions have differently named fields
    if ( preg_match( '{<input.*?name="articletitle"\s}', $data ) ) {
      $keys = array_keys( $params );
      foreach ( $keys as $k ) {
        $params[ "article$k" ] = $params[ $k ];
        unset( $params[ $k ] );
      }

      # Stupid form design :/
      if ( strpos( $data, '<input type=submit name=submit value="Submit My Article"' ) ) {
        unset( $params[ 'publish' ] );
        $params[ 'submit' ] = "Submit My Article";
      }
      else {
        $params[ "submit" ] = "Submit";
      }
    }

    $excerpt_length = 55;

    # Site specific overrides :/
    if ( in_array( $this->root, array( 'http://www.relationshiprepair.net/articlems' ) ) ) {
      $excerpt_max_chars = 190;
    }
    elseif ( preg_match( '{<textare.*?name="((?:article)?summary)"}', $data, $name_match ) ) {
      $fieldname = $name_match[1];

      if ( $this->use_empty_summary ) {
        $text = "";
      }
      else {
        $text = strip_shortcodes( $this->article->content );

        $text = str_replace(']]>', ']]&gt;', $text);
        $text = strip_tags($text);

        $words = preg_split("/[\n\r\t ]+/", $text, $excerpt_length + 1, PREG_SPLIT_NO_EMPTY);
        if ( count($words) > $excerpt_length ) {
          array_pop($words);
        }
        $text = implode(' ', $words);

        if ( $excerpt_max_chars ) $text = substr( $text, 0, $excerpt_max_chars );
      }

      $params[ $fieldname ] = $text;
    }

    if ( strpos( $data, '<input name="agreecheck" type="checkbox"' ) !== false ) {
      $params[ 'agreecheck'] = 'on';
    }

    $this->submit_params = $params;

    # Some forms seem to have captchas on the submit page too
    $this->on_captcha_solved_method = "submit_captcha_solved";
    if ( preg_match( '/<img\s+[^>]*src=".*?visual-captcha.php"[^>]*>/', $data ) ) {
      return $this->get_builtin_captcha( $info, $data );
    }
    elseif( preg_match('{"http://api.recaptcha.net}', $data) ) {
      return $this->get_recaptcha( $info, $data );
    }
    $this->do_submit_post();
  }

  function submit_captcha_solved( $captcha ) {

    $this->do_submit_post( $captcha );
  }

  function do_submit_post( $captcha = NULL) {
    # Make sure we don't loop
    if ( ++$this->submit_attempt > 3 ) {
      dab_status( "{$this->root} 3rd submission attempt" );
      return;
    }

    $params = $this->submit_params;
    $this->set_captcha_fields( $params, $captcha );

    $params = http_build_query( $params );

    dab_debug( "{$this->root}: submit params " . print_r( $params, true ) );
    dab_status( "{$this->root} Posting submission page" );
    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $params,
      CURLOPT_REFERER => "{$this->root}/author/submit.php",
      CURLOPT_URL => "{$this->root}/submit/",
      CURLOPT_COOKIE => $this->cookies()
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_submit_article_post' ),
      array( $this, 'abort' )
    );
  }

  protected static $SITES_WITHOUT_PRINTVIEW = array(
    'http://www.links7.com',
    'http://www.diyresource.com',
    'http://www.deadvip.com',
    'http://www.thinkgreenarticles.com',
    'http://www.boat-dealers.us/articles',
    'http://www.publishingdaily.com',
    'http://www.crikeydirectory.com/pages',

    'http://articleknowledgebase.com'
  );

  function on_submit_article_post( $info, $data ) {
    $this->cookies( $info[ 'cookies' ] );

    if ( preg_match_all( '/<div\s+style="color:red"[^>]*>\s*(.*?)\s*<\/div>/s', $data, $matches ) > 0 ||
         preg_match_all('/<font color=red>\s*(.*?)\s*<\/font>/s', $data, $matches) > 0 ) {

      if ( in_array( 'You must enter the verification code', $matches[1] ) ) {
        dab_debug("captcha {$this->captcha_id} failed");
        DAB_CaptchaService::instance()->report_failure( $this->captcha_id );

        # Try again.
        return $this->retry( $info, $data );
      }
      if ( in_array( 'Summary is too short.', $matches[1] ) ||
           in_array( 'Summary is too long.', $matches[1] ) ||
           in_array( 'Invalid Summary', $matches[1] ) )
      {
        # Try again without a summary in this case.
        $this->use_empty_summary = true;
        return $this->retry( $info, $data );
      }
      dab_debug("{$this->root} " .print_r( $matches, true ) );
    }
    # "<div class=\"ui-state-error ui-corner-all\" style=\"margin-bottom: 8px; padding: 0.5em 0.7em 0.5em 0.7em;\">\r\n\t\t\t\t<p><span class=\"ui-icon ui-icon-alert\" style=\"float: left; margin-right: .3em;\"></span>\r\n\t\t\t\tAn article with the same title has already been submitted to this category.</p>\r\n\t\t\t</div>\r\n
    elseif ( preg_match_all( '{<div class=".*?ui-state-error.*?>\s*<p>(?:<span class=".*?icon.*"></span>\s*)?(.*?)</p>\s*</div>}s', $data, $matches) ) {
      $matches[0] = ''; # Remove the text we matched over in debug prints
      dab_debug("{$this->root} " .print_r( $matches, true ) );
    }

    if ( preg_match( '{/edit/\?.*?id=(\d+).*?$}', $info['url'], $match ) ||
         preg_match( '{submit/\?msg=\d+&id=(\d+)}', $info['url'], $match ) ) {
      $url = NULL;
      if ( !in_array( $this->root, self::$SITES_WITHOUT_PRINTVIEW ) )
        $url = "{$this->root}/view/printview-{$match[1]}.html";
      $this->record_submission_finished( $url, $info );
    }
    elseif( preg_match( '/Your Article has been successfully added|submitted/i', $data) ) {
      # Older versions don't incude the id - lame
      return $this->my_articles( $info, $data );
    }
    else {
      dab_debug( $data );
      $this->record_submission_finished( NULL, $info, $data, $matches );
    }
  }

  function my_articles( $info, $data ) {
    dab_status( "{$this->root} getting list of submitted articles" );
    $curl = dab_curl( array(
      CURLOPT_REFERER => $info[ 'url' ],
      CURLOPT_URL => "{$this->root}/my_articles/",
      CURLOPT_COOKIE => $this->cookies()
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_my_articles_get' ),
      array( $this, 'abort' )
    );
  }

  function on_my_articles_get( $info, $data ) {

    # Find the edit links to get the highest id.
    $title = preg_quote( $this->article->title );
    preg_match_all( '{\d+:\s*(?:<a[^>]*?>)?' . $title . '.*?<a href=([\'"]).*?edit/\?id=(\d+)\1>}s', $data, $edit_ids );
    dab_debug ("edit_ids: " . var_export( $edit_ids, true ) );
    sort( $edit_ids[2], SORT_NUMERIC );

    $id = end( $edit_ids[2] );

    dab_debug("id is $id - checking for view link to it: \"/article.php?id={$id}/\"");

    if ( $id && !in_array( $this->root, self::$SITES_WITHOUT_PRINTVIEW ) )
      $url = "{$this->root}/view/printview-$id.html";
    $this->record_submission_finished( $url, $info );
  }

  function login( $info, $data ) {
    $this->cookies( $info['cookies'] );

    # There might be a captcha on the login form.
    $this->on_captcha_solved_method = "do_login";
    if ( preg_match( '/<img\s+[^>]*src=".*?visual-captcha.php"[^>]*>/', $data ) ) {
      $this->get_builtin_captcha( $info, $data);
    }
    elseif( preg_match('{"http://api.recaptcha.net}', $data) ) {
      $this->get_recaptcha( $info, $data );
    }
    else {
      $this->do_login( NULL );
    }
  }

  function set_captcha_fields( &$params, $captcha ) {
    switch( $this->captcha_type ) {
    case 'recaptcha':
      $params[ 'recaptcha_challenge_field' ] = $captcha;
      $params[ 'recaptcha_response_field' ] = 'manual_challenge';
      break;
    case 'builtin':
      $params[ 'image_verification' ] = $captcha;
      break;
    }
  }

  function do_login( $captcha ) {
    # Make sure we don't loop
    if ( ++$this->login_attempt > 3 ) {
      $this->login_failed();
      return;
    }

    $params = array(
      'user' => $this->username,
      'pass' => $this->password,
      'submit' => "Submit"
    );

    dab_debug( "{$this->root} captcha type is {$this->captcha_type}" );

    $this->set_captcha_fields( $params, $captcha );

    $params = http_build_query( $params );

    dab_debug( "{$this->root} Posting login form $params" );
    dab_status( "{$this->root} logging in" );

    $url = "{$this->root}/login/";

    if ( $this->root == "http://www.articlescolumn.com" )
      $url= "{$this->root}/getin/";

    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $params,
      CURLOPT_REFERER => $url,
      CURLOPT_URL => $url,
      CURLOPT_COOKIE => $this->cookies()
    ) );

    DAB_MultiCurl::add(
      $curl,
      array( $this, 'on_login_post' ),
      array( $this, 'abort' )
    );
  }

  function on_login_post( $info, $data ) {
    $this->cookies( $info['cookies'] );

    if ( preg_match( '{login/?$}', $info['url'] ) && strpos( $data, 'You have successfully logged in.' ) === false ) {
      if ( preg_match_all( '/<div\s+style="color:red"[^>]*>\s*(.*?)\s*<\/div>/s', $data, $matches ) > 0 ||
           preg_match_all('/<font color=red>\s*(.*?)\s*<\/font>/s', $data, $matches) > 0 ) {
        $this->notes .= "\n---\n" .print_r( $matches[1], true );
        $this->errors = $matches[1];
      }
      else {
        dab_debug( "{$this->root} Uhoh - stayed on login page!");
        dab_debug( $data );
      }
      $this->login_failed();
      return;
    }
    dab_debug( "{$this->root}: login_succeded" );
    $this->login_succeded();

    $this->get_submit_article_page( $info );
  }

  function get_categories( $info, $html ) {
    # <select name="category">...\n...</select>
    $opts = $this->get_select_options( "(?:article)?category", $html );

    $cats = array();

    # [Top]
    # &nbsp;&nbsp;Arts & Entertaiment
    # &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Celebrities

    # Some have [Top], some don't

    $parent_cats = array();

    preg_match_all('/<option[^>]+value=(["\'])(.*?)\1[^>]*>(.*?)\s*(?=<\/option>|<option|$)/sD',  $matches[1], $matches);
    foreach( $opts as $cat => $val ) {
      # Ignore the [Top] if there is one
      if ( $val == 0 ) continue;

      $cat = html_entity_decode( $cat, ENT_COMPAT, 'UTF-8' );
      # Strip of the leading two.
      $cat = preg_replace( '/^(\xC2\xA0){2}/', '', $cat);

      # Count how deep we are
      $depth = 1 + preg_match_all( '/(\xC2\xA0){4}/', $cat, $tmp );

      $cat = preg_replace( '/^(\xC2\xA0)*/', '', $cat);

      if ( $depth ) {
        while ( count( $parent_cats ) >= $depth ) {
          array_pop( $parent_cats );
        }
        array_push( $parent_cats, $cat );
        $cat = implode(' -- ', $parent_cats);
      }
      else {
        $parent_cats = array( $cat );
      }

      $cats[ $cat ] = $val;
    }

    return $cats;
  }
}
