<?php

add_action( 'wp_ajax_dab_test_captcha', 'dab_test_captcha' );

function dab_test_captcha() {
  error_reporting( E_ERROR );

  header("Content-Type: application/json");
  header("Cache-Control: no-cache");
  header("Pragma: no-cache");

  $obj = DAB_CaptchaService::build( $_POST[ 'provider' ], $_POST[ 'user' ], $_POST[ 'pass' ] );

  try {
    $credits = $obj->get_balance();
    echo '{"balance": "' . dab_make_json_safe( $credits ) . '"}';
  }
  catch (Exception $e) {
    echo '{"error": "' . dab_make_json_safe( $e->getMessage() ) . '"}';
  }
  exit;
}


class DAB_CaptchaService {
  protected $user;
  protected $pass;

  static protected $instance = NULL;

  static function build( $provider, $user, $pass ) {
    switch ( $provider ) {
      case "Bypass CAPTCHA":
        return new DAB_Bypass_CAPTCHA( $user );
      case "Decaptcher":
        return new DAB_Decaptcher( $user, $pass );
      case "Death by Captcha":
        return new DAB_DeathByCatpcha( $user, $pass );
    }
  }

  static function instance() {
    if (self::$instance === NULL) {
      $config = get_option( DAB_OPTIONS_KEY, array() );

      self::$instance = self::build( $config[ 'decaptcha_provider'], $config[ 'decaptcha_user' ], $config[ 'decaptcha_pass' ] );
    }
    return self::$instance;
  }

  protected function __construct($user, $pass = null) {
    $this->user = $user;
    $this->pass = $pass;
  }

  protected function temp_file() {
    // Some blogs can't write to /tmp (or the system temp dir), so fall back to
    // the wordpress get_temp_dir
    $tmpfile = @tempnam( '', "dab-captcha-" );
    if (!$tmpfile)
      $tmpfile = tempnam( get_temp_dir(), "dab-captcha-" );
    return $tmpfile;
  }
}

class DAB_Bypass_CAPTCHA extends DAB_CaptchaService {
  protected static $url = "http://bypasscaptcha.com/upload.php";
  protected static $report_url = "http://bypasscaptcha.com/check_value.php";

  function get_balance() {
    $curl = dab_curl( array(
      CURLOPT_URL => 'http://bypasscaptcha.com/ex_left.php',
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => http_build_query( array( 'key' => $this->user ) )
    ) );

    $res = curl_exec( $curl );

    $out = $this->parse_response( $res );

    if ( isset( $out[ 'Error' ] ) ) {
      throw new Exception( $out[ 'Error' ] );
    }
    return $out['Left'] . ' submissions';
  }

  function solve( $image_data, $success, $failure = NULL ) {
    if ( !is_array( $success ) ) {
      $receiver = $success;
      $failure = array( $failure, 'on_captcha_failed' );
      $success = array( $success, 'on_captcha_solved' );
    }

    # The upload might take a while, so do it next request
    DAB_MultiCurl::enqueue( array( $this, 'post_captcha' ), array( $image_data, $success, $failure ) );
  }

  function post_captcha( $image_data, $success, $failure ) {

    $data = array(
      'file' => base64_encode( $image_data ),
      'submit' => "Submit",
      'gen_task_id' => 1,
      'key' => $this->user,
      'base64_code' => 1,
      'vendor_key' => '26ee8f5f8d3a3d733876c291fb77f662d405464d'
    );

    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $data,
      CURLOPT_URL => self::$url,
      CURLOPT_TIMEOUT => 26
    ) );
    DAB_MultiCurl::add( $curl, array( $this, 'on_response' ), null, array( $success, $failure ) );
  }

  function parse_response( $data ) {
    # Pull out the info from the response.
    $out = array();
    foreach ( explode( "\n", $data ) as $line ) {
      $a = explode( " ", $line, 2 );
      $out[ $a[0] ] = $a[1];
    }
    return $out;
  }

  function on_response( $info, $data, $success, $failure ) {

    $out = $this->parse_response( $data );

    # Call the right callback
    $err = $out[ 'Error' ];
    if ($out['Value'] == 'Incorrect_captcha_image_and_you_will_not_be_charged' ) {
      $err = $out['Value'];
    }

    if( isset( $err ) ) {
      if ($failure) {
        call_user_func( $failure, $err );
      }
    }
    else {
      call_user_func(
        $success,
        $out['Value'],
        $out['TaskId']
      );
    }
  }

  # Tell the captcha service that this decoding didn't work.
  function report_failure( $task_id ) {
    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => array(
        'cv' => 0,
        'task_id' => $task_id,
        'key' => $this->user,
        'submit' => "Submit"
      ),
      CURLOPT_URL => self::$report_url
    ) );
    DAB_MultiCurl::add( $curl );
  }
}

class DAB_Decaptcher extends DAB_CaptchaService {
  protected static $url = "http://poster.decaptcher.com/";

  function get_balance() {
    $fields = array(
      'function' => 'balance',
      'username' => $this->user,
      'password' => $this->pass
    );

    $curl = dab_curl( array(
      CURLOPT_URL => self::$url,
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => http_build_query( $fields )
    ) );

    $res = curl_exec( $curl );

    if ( $res == "" ) {
      # probably. Doesn't seem to be any error condition other than this to
      # detect
      throw new Exception( "Invalid username/password" );
    }

    return "$" . $res;
  }

  function solve( $image_data, $success, $failure = NULL ) {
    if ( !$failure && !is_array( $success ) ) {
      $receiver = $success;
      $failure = array( $receiver, 'on_captcha_failed' );
      $success = array( $receiver, 'on_captcha_solved' );
    }

    $tmpfile = $this->temp_file();
    $fh = fopen( $tmpfile, "wb" );
    fwrite( $fh, $image_data );

    # The upload might take a while, so do it next request
    DAB_MultiCurl::enqueue( array( $this, 'post_captcha' ), array( $tmpfile, $success, $failure ) );

  }

  function post_captcha( $tmpfile, $success, $failure ) {

    $fields = array(
      'function' => 'picture2',
      'username' => $this->user,
      'password' => $this->pass,
      'pict_to' => 0,
      'pict_type' => 0,
      'pict' => "@$tmpfile"
    );

    $curl = dab_curl( array(
      CURLOPT_URL => self::$url,
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $fields,
      CURLOPT_TIMEOUT => 26
    ) );

    DAB_MultiCurl::add( $curl, array( $this, 'on_response' ), array( $this, 'on_response' ), array( $tmpfile, $success, $failure ) );
  }

  function on_response( $info, $data, $tmpfile, $success, $failure ) {

    unlink( $tmpfile );

    # ResultCode|MajorID|MinorID|Type|Timeout|Text
    $state = explode( '|', $data );

    $err = $state[0];
    if( $err != "0" ) {
      if ($failure) {
        call_user_func( $failure, $err );
      }
    }
    elseif ( strlen( $state[5] ) ) {
      call_user_func(
        $success,
        $state[5],
        $state[1] . '|' . $state[2]
      );
    }
  }

  function report_failure( $task_id ) {
    $ids = explode( '|',  $task_id );
    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => array(
        'function' => 'picture_bad2',
        'username' => $this->user,
        'password' => $this->pass,
        'major_id' => $ids[0],
        'minor_id' => $ids[1]
      ),
      CURLOPT_URL => self::$url
    ) );
    DAB_MultiCurl::add( $curl );
  }
}

class DAB_DeathByCatpcha extends DAB_CaptchaService {
  protected static $url = "http://api.deathbycaptcha.com/api";
  // protected static $url = "http://localhost:8888/api";

  function get_balance() {
    $fields = array(
      'username' => $this->user,
      'password' => $this->pass
    );

    $curl = dab_curl( array(
      CURLOPT_URL => self::$url . '/user',
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => http_build_query( $fields )
    ) );

    $res = curl_exec( $curl );
    $info = curl_getinfo( $curl );

    # Yay! REST
    if ( $info['http_code'] == 403 )
      throw new Exception( "Invalid username/password" );

    # Parse query=str&params=
    parse_str( $res, $out );
    return sprintf("$%.02f", $out['balance']/100.0 );
  }

  function solve( $image_data, $success, $failure = NULL ) {
    if ( !$failure && !is_array( $success ) ) {
      $receiver = $success;
      $failure = array( $receiver, 'on_captcha_failed' );
      $success = array( $receiver, 'on_captcha_solved' );
    }

    # The upload might take a while, so do it next request
    DAB_MultiCurl::enqueue( array( $this, 'post_captcha' ), array( $image_data, $success, $failure ) );
  }

  function post_captcha( $image_data, $success, $failure ) {

    $fields = array(
      'username' => $this->user,
      'password' => $this->pass,
      'captchafile' => 'base64:' . base64_encode( $image_data )
    );
    dab_debug( "DeatchByCaptcha#params: " . var_export( $fields, true ) );

    $curl = dab_curl( array(
      CURLOPT_URL => self::$url . '/captcha',
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => $fields,
      CURLOPT_HTTPHEADER => array( 'Expect: ' )
    ) );

    DAB_MultiCurl::add( $curl, array( $this, 'on_response' ), array( $this, 'on_response' ), array( $success, $failure, time() ) );
  }

  function on_response( $info, $data, $success, $failure, $last_poll, $start_time ) {

    dab_debug( "DeatchByCaptcha#on_response: info=" . var_export( $info, true ) );
    dab_debug( "DeatchByCaptcha#on_response: data=" . var_export( $data, true ) );

    # Parse query=str&params=
    parse_str( $data, $params );

    if ($info[ "http_code" ] != 200 ) {
      if ( $failure )
        call_user_func( $failure, $data );
    }
    elseif ( $params[ "text" ] != "" ) {
      call_user_func( $success, $params[ "text" ], $params[ "captcha" ] );
    }
    else {
      if ( is_null( $start_time ) )
        $start_time = $last_poll;

      if ( ( $last_poll - $start_time ) > 60 ) {
        # Took too long - giving up
        dab_status( "DeathByCaptcha too too long for id: {$params['captcha']}." );
        if ( $failure )
          call_user_func( $failure, $data );
        return;
      }

      # poll again.
      DAB_MultiCurl::enqueue(
        array( $this, 'poll' ),
        array( $params[ "captcha" ], $success, $failure, $last_poll, $start_time )
      );
    }

  }

  function poll( $task_id, $success, $failure, $last_poll, $start_time ) {
    $now = time();

    if ($now - $last_poll < 2) {
      # Enqueue to poll later
      DAB_MultiCurl::enqueue(
        array( $this, 'poll' ),
        array( $task_id, $success, $failure, $last_poll, $start_time )
      );
    }
    else {
      $curl = dab_curl( array( CURLOPT_URL => self::$url . "/captcha/$task_id" ) );

      DAB_MultiCurl::add(
        $curl,
        array( $this, 'on_response' ),
        array( $this, 'on_response' ),
        array( $success, $failure, $now, $start_time )
      );
    }
  }

  function report_failure( $task_id ) {

    $curl = dab_curl( array(
      CURLOPT_POST => true,
      CURLOPT_POSTFIELDS => http_build_query( array(
        'username' => $this->user,
        'password' => $this->pass
      ) ),
      CURLOPT_URL => self::$url . "/captcha/$task_id/report"
    ) );
    DAB_MultiCurl::add( $curl );
  }

}
