<?php
/*
Plugin Name: Digi Article Blaster
Plugin URI: http://www.digiarticleblaster.com/
Description: Digi Article Blaster spins and submits your posts to hundreds of articles directories for you.
Version: 1.3.5
Author: DigiResults
Author URI: http://www.digiresults.com/
*/

define( 'DAB_VERSION', '1.3.5' );
define( 'DAB_OPTIONS_KEY', 'digiarticleblaster_options' );
define( 'DAB_SITE_LIST_VERSION_KEY', 'digiarticleblaster_site_list_verison' );
define( 'DAB_ADMIN_URL', admin_url( 'admin.php?page=' . DAB_OPTIONS_KEY ) );
define( 'DAB_PLUGIN_BASE', substr( plugin_basename( __FILE__ ), 0, strpos( plugin_basename( __FILE__ ), '/' ) ) );
define( 'DAB_PLUGIN_URL', WP_PLUGIN_URL . '/' . DAB_PLUGIN_BASE );
define( 'DAB_DB_VERSION', '6' );
define( 'DAB_META_KEY', '_dab_blasted' );
define( 'DAB_ARTICLE_URLS_KEY', '_dab_article_urls' );
define( 'DAB_ARTICLE_SUBMISSION_KEY', '_dab_article_submisisons' );

include 'lib/digipowertools.php';

// admin pages
add_action( 'admin_menu', create_function( '', "require( 'admin-pages.php' );" ) );
add_action( 'init', 'dab_register_scripts', 1 );


// WP Ad Rotator - move it to a different hook so it doesn't happen in init
// which we can't affect
if ( function_exists( 'wpadr_init' ) ) {
  remove_action('init', 'wpadr_init');
  add_action('admin_enqueue_scripts', 'wpadr_init');
}

// installation
register_activation_hook( __FILE__, 'dab_install' );

// Get URLs where the article can be viewed
add_action( 'wp_ajax_dab_view_article_urls', 'dab_view_article_urls' );

// Debug/meta hook
add_action( 'wp_ajax_dab_view_post_state', 'dab_view_post_state' );
add_action( 'wp_ajax_dab_view_account_state', 'dab_view_account_state' );
add_action( 'wp_ajax_dab_nuke_state', 'dab_nuke_state' );
add_action( 'wp_ajax_dab_nuke_accounts', 'dab_nuke_accounts' );
add_action( 'wp_ajax_dab_get_blasted_posts', 'dab_get_blasted_posts' );



// make sure the default config is patched to current
dab_update_default_config();


require_once( 'article-directories.php' );


function dab_register_scripts() {
  wp_register_script(
    'jquery-ui-widget',
    plugins_url( 'jquery-ui/jquery-ui-1.8.13.widget.js', __FILE__ ),
    array( 'jquery-ui-core', 'jquery'),
    '1.8.13'
  );
  wp_register_script(
    'jquery-ui-progressbar',
    plugins_url( 'jquery-ui/jquery-ui-1.8.13.progressbar.js', __FILE__ ),
    array( 'jquery-ui-widget', 'jquery-ui-core', 'jquery'),
    '1.8.13'
  );
  wp_register_style(
    'jquery-ui-lightness',
    plugins_url( 'jquery-ui/jquery-ui-1.8.13.lightness.css', __FILE__ ),
    array(),
    '1.8.13'
  );
  wp_register_style(
    'jquery-ui-progressbar',
    plugins_url( 'jquery-ui/jquery-ui-1.8.13.progressbar.css', __FILE__ ),
    array(),
    '1.8.13'
  );

  wp_register_style(
    'dab_admin',
    plugins_url( 'styles/admin.css', __FILE__ ),
    array()
  );

  # Too many scripts futz with this, so keep and restore it on our pages.
  # WP Ad Rotator, VEZImporter are two
  global $dab_original_wp_jquery, $wp_scripts;
  $dab_original_wp_jquery = $wp_scripts->registered['jquery'];
}

function dab_update_db( $current_ver = 0 ) {
  global $wpdb;
  require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

  dbDelta( array(
    "CREATE TABLE {$wpdb->prefix}dab_accounts (
      id INT AUTO_INCREMENT NOT NULL,
      url VARCHAR( 255 ) NOT NULL UNIQUE,
      type VARCHAR( 32 ) NOT NULL,
      username VARCHAR( 255 ),
      password VARCHAR( 255 ),
      working tinyint(1) DEFAULT 0 NOT NULL,
      cookies TEXT,
      notes TEXT,
      failed_logins INT DEFAULT 0 NOT NULL,
      PRIMARY KEY id ( id )
    );",
    "CREATE TABLE {$wpdb->prefix}dab_state (
      id INT AUTO_INCREMENT NOT NULL,
      sites_total INT,
      sites_left INT,
      state LONGBLOB,
      post_id INT NULL,
      PRIMARY KEY id ( id )
    )"
  ) );

  global $wpdb;
  if ( get_option( DAB_SITE_LIST_VERSION_KEY, 0) == 0 &&
       $wpdb->get_var( "SELECT count(*) FROM {$wpdb->prefix}dab_accounts" ) > 0 )
  {
    add_option( DAB_SITE_LIST_VERSION_KEY, 1);
  }

  if ( $current_ver < 5 ) {
    # Remove accounts to these sites.
    $wpdb->query("DELETE FROM {$wpdb->prefix}dab_accounts
                        WHERE url IN ( 'http://101info.org/pcimarticles', 'http://draftingservice.us/drafting', 'http://www.christarena.com' )");
  }
  update_option( 'dab_db_version', DAB_DB_VERSION );

}

if (get_option( 'dab_db_version') != DAB_DB_VERSION ) {
  dab_update_db( get_option( 'dab_db_version') );
}

function dab_install() {
  dab_update_db();

  $author = wp_get_current_user();
  dab_update_default_config( array(
    'penname' => $author->display_name,
    'email' => $author->user_email,
    'useragent' => $_SERVER["HTTP_USER_AGENT"]
  ) );

}

function dab_update_default_config( $override = array() ) {
  $defaults = array_merge( array(
  ), $override );

  $config = get_option( DAB_OPTIONS_KEY );
  if ( !is_array( $config ) ) $config = array();

  $config = array_merge( $defaults, $config );
  update_option( DAB_OPTIONS_KEY, $config );

  global $dab_user_agent;
  $dab_user_agent = $config[ 'useragent' ];

}

function dab_debug( $str ) {
  if ( !isset( $_GET['debug'] ) ) return;

  if ( !is_string( $str ) ) $str = var_export( $str, true );

  error_log( $str );
  if ($_GET['debug'] == 'capture' ) {
    global $dab_debug_capture;
    if (!isset( $dab_debug_capture ) ) $dab_debug_capture = array();
    $dab_debug_capture[] = $str;
  }
  else {
    echo "<pre>" . htmlspecialchars( $str ) . "</pre>\n";
  }
}

function dab_status( $str ) {
    global $dab_status_capture;
    if (!isset( $dab_status_capture ) ) $dab_status_capture = array();
    $dab_status_capture[] = $str;
}

// debug function
function dab_view_post_state() {
  global $wpdb;

  echo "<p>DAB version is " . DAB_VERSION . "</p>";
  echo '<p>PHP version is ' . phpversion() . "</p>";
  echo '<p>MySQL version is ' . $wpdb->get_var( 'SELECT VERSION()' ) . "</p>";
  echo '<hr />';


  $post_id = $_GET['post'];
  $blasted = @unserialize( get_post_meta( $post_id, DAB_META_KEY, true ) );

  if ( !$blasted ) exit;

  foreach( $blasted as $site_id => $data ) { ?>
    <p>Site Id: <?php echo $site_id ?></p>
    <p>Final url: <?php echo htmlspecialchars( $data['info'][ 'url' ] ) ?></p>
    <p>Alerts: <?php echo htmlspecialchars( var_export( $data['alerts'], true ) ) ?></p>
<?php
    if ( $data[ 'data' ] || true ) {
      $blob = gzinflate( $data[ 'data' ] );
?>
    <p>HTML displayed</p>
    <textarea rows="20" cols="100"><?php echo htmlspecialchars( $blob ) ?></textarea>
<?php
    }
?>
  <hr />
<?php
  }
  exit;
}

// debug function
function dab_view_account_state() {
  global $wpdb;

  $sites = DAB_Account::all_sites( "id" );

  echo "<p>DAB version is " . DAB_VERSION . "</p>";
  echo '<p>PHP version is ' . phpversion() . "</p>";
  echo '<p>MySQL version is ' . $wpdb->get_var( 'SELECT VERSION()' ) . "</p>";
  echo '<p>There are ' . count($sites) . ' account rows</p>';
  echo '<p>There are ' . DAB_Account::num_working_sites() . ' working accounts</p>';
  echo '<hr />';


  foreach( $sites as $site ) { ?>
    <pre><?php echo htmlspecialchars( var_export( $site, true ) ) ?></pre>
    <p>Site Id: <?php echo $site->id ?></p>
    <p>Url: <a href="<?php echo $site->root ?>"><?php echo $site->root ?></a></p>
    <p>State: <?php echo $site->working ?></p>
    <p>Failed attempts: <?php echo $site->failed_logins ?></p>
    <p>Username: <?php echo $site->username ?></p>
    <p>Password: <?php echo $site->password ?></p>
    <p>Notes
    <textarea rows="20" cols="100"><?php echo htmlspecialchars( $site->notes ) ?></textarea></p>
  <hr />
<?php
  }
  exit;
}

function dab_nuke_state() {
  global $wpdb;
  echo "<p>DAB version is " . DAB_VERSION . "</p>";
  echo '<p>PHP version is ' . phpversion() . "</p>";
  echo '<p>MySQL version is ' . $wpdb->get_var( 'SELECT VERSION()' ) . "</p>";

  $wpdb->show_errors();
  echo $wpdb->query( "DELETE FROM {$wpdb->prefix}dab_state");

}

function dab_nuke_accounts() {
  global $wpdb;
  echo "<p>DAB version is " . DAB_VERSION . "</p>";
  echo '<p>PHP version is ' . phpversion() . "</p>";
  echo '<p>MySQL version is ' . $wpdb->get_var( 'SELECT VERSION()' ) . "</p>";

  if ( !isset( $_GET['do'] ) || $_GET['do'] != '1' ) return;

  $wpdb->show_errors();
  echo $wpdb->query( "DELETE FROM {$wpdb->prefix}dab_state");
  echo $wpdb->query( "DELETE FROM {$wpdb->prefix}dab_accounts");
  exit;
}

function dab_view_article_urls() {
  if ( $_GET['debug'] ) $_GET['debug'] = 'capture';

  # Silence warnings too!
  error_reporting( E_ERROR );

  header("Content-Type: application/json");
  header("Cache-Control: no-cache");
  header("Pragma: no-cache");

  $post_id = $_REQUEST['post'];
  $urls = get_post_meta( $post_id, DAB_ARTICLE_URLS_KEY, false );

  $count = get_post_meta( $post_id, DAB_ARTICLE_SUBMISSION_KEY, true );

  if ( !is_numeric( $count ) ) $count = 0;

  if ( $count < count( $urls ) ) $count = count( $urls );

  $num = count($urls);
  $last = array_pop( $urls );

  echo "{ \"count\": $count, \"num_published\": $num, \"urls\": [\n";
  foreach ( $urls as $url ) {
    echo '"' . addcslashes( $url, "\t\n\r\"\\" ) . "\",\n";
  }
  if ( $last )
    echo '"'. addcslashes( $last, "\t\n\r\"\\" ) . "\"\n";
  echo "] }";
  exit;
}

function dab_get_blasted_posts() {
  global $wpdb;
  echo "<p>DAB version is " . DAB_VERSION . "</p>";
  echo '<p>PHP version is ' . phpversion() . "</p>";
  echo '<p>MySQL version is ' . $wpdb->get_var( 'SELECT VERSION()' ) . "</p>";
  echo '<hr />';

  $wpdb->show_errors();
  foreach ( $wpdb->get_col( "SELECT DISTINCT post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = \"_dab_article_urls\"" ) as $id) {
    echo "<p>$id</p>";
  }
  exit;
}
