<?php

add_action( 'wp_ajax_dab_do_work', 'dab_do_work' );

function dab_do_work() {
  global $dab_debug_capture, $dab_status_capture;

  if ( $_GET['debug'] ) $_GET['debug'] = 'capture';

  # Continue running even if the user connection aborts. Mainly try to continue
  # processing and make sure we release the work lock
  ignore_user_abort( true );

  # Silence warnings too!
  error_reporting( E_ERROR );

  header("Content-Type: application/json");
  header("Cache-Control: no-cache");
  header("Pragma: no-cache");

  $started = gettimeofday(true);
  global $wpdb;
  $lock = $wpdb->get_var("SELECT get_lock('digiarticleblaster_curl', 0)");

  # Set the Connection to stay alive 10 minutes, and failing that try least 45 seconds
  $timeout = $wpdb->get_var("SHOW SESSION VARIABLES LIKE 'wait_timeout'", 1);
  if ($timeout < 600 )
    $wpdb->query( "SET SESSION wait_timeout=600" );
  $timeout = $wpdb->get_var("SHOW SESSION VARIABLES LIKE 'wait_timeout'", 1);
  if ($timeout < 45 )
    $wpdb->query( "SET SESSION wait_timeout=45" );

  $initial_work = DAB_MultiCurl::total_work_available();

  # If we couldn't obtain the lock something else is already doing some work.
  if (!$lock) {
    ?>
{ "work_done": 0,
  "total_work": <?php echo $initial_work ? $initial_work : 0 ?>,
  "more_to_do": true,
  "dab_debug_capture": [
    "unable to obtain digiarticleblaster_curl lock"
  ],
  "log": []
 }
<?php
    die;
  }

  if ( isset( $_GET['do'] ) ) {
    # Enable output buffering to shut up other plugins that are somehow printing stuff here
    ob_start();
    $multi = DAB_MultiCurl::instance();

    $multi->exec();

    $multi->store();
    ob_end_clean();
  }

  dab_debug('done work chunk in ' . (gettimeofday(true) - $started) );
  $remaining_work = DAB_MultiCurl::total_work_available();

  # MySQL time outs cause these :/
  if (is_null( $remaining_work ) ) $remaining_work = $initial_work;

  $work_done = $initial_work - $remaining_work;
  if ( $work_done < 0 ) $work_done = 0;

  if (!isset( $dab_debug_capture ) ) $dab_debug_capture = array();
  if (!isset( $dab_status_capture ) ) $dab_status_capture = array();

  ?>
{ "work_done": <?php echo $work_done ?>,
  "total_work": <?php echo $initial_work ? $initial_work : 0 ?>,
  "more_to_do": <?php echo $remaining_work ? "true" : "false" ?>,
  "dab_debug_capture": [
    "<?php echo implode("\",\n    \"", array_map( 'dab_make_json_safe', $dab_debug_capture ) ) ?>"
  ],
  "log": [
    "<?php echo implode("\",\n    \"", array_map( 'dab_make_json_safe', $dab_status_capture ) ) ?>"
  ]
}
<?php

  $wpdb->query("SELECT release_lock('digiarticleblaster_curl')");

  die;
}

function dab_make_json_safe( $x ) {
  return addcslashes($x, "\t\n\r\"\\");
}


# Helper function to create curl objects
function dab_curl( $opts ) {
  global $dab_user_agent;
  $curl = curl_init();

  $defaults = array(
    CURLOPT_RETURNTRANSFER => true,
    CURLOPT_FOLLOWLOCATION => true,
    CURLOPT_TIMEOUT => 20
  );

  # Lets make it look like the admin's browser for giggles
  if ( isset( $dab_user_agent ) ) $defaults[ CURLOPT_USERAGENT ] = $dab_user_agent;
  if ( is_array( $opts[CURLOPT_COOKIE] ) )
    $opts[CURLOPT_COOKIE] = implode( '; ', $opts[CURLOPT_COOKIE] );

  dab_debug( $opts[CURLOPT_URL] . ' cookies:' . print_r( $opts[CURLOPT_COOKIE], true ) );
  curl_setopt_array( $curl, $opts + $defaults );

  return $curl;
}


class DAB_MultiCurl implements Serializable{

  protected $multi_handle;
  protected $success_callbacks;
  protected $failure_callbacks;
  protected $running;

  static protected $instance = NULL;

  static function instance() {
    global $wpdb;

    if (self::$instance === NULL ) {
      # Get some work to do
      $stored = $wpdb->get_row("SELECT id,state,post_id from {$wpdb->prefix}dab_state ORDER BY id LIMIT 1");

      if ($stored) {
        self::$instance = unserialize( $stored->state );
        self::$instance->id = $stored->id;
        self::$instance->post_id = $stored->post_id;
      }
      else {
        self::$instance = new self();
      }
    }
    return self::$instance;
  }

  static function new_batch( $post_id = NULL ) {
    return new self( $post_id );
  }

  static function add( ) {
    # Proxy to the instance.

    # Stupidly you have to store it in a local variable first else php 5.0.x complains.
    $args = func_get_args();
    call_user_func_array( array( self::instance(), "_add" ), $args );
  }

  static function enqueue() {
    # Proxy to the instance.
    $args = func_get_args();
    call_user_func_array( array( self::instance(), "_enqueue" ), $args );
  }

  static function work_available() {
    global $wpdb;

    # More work in this batch
    if ( !self::instance()->is_done() ) return true;

    # Or more batches
    $stored = $wpdb->get_row("SELECT id from {$wpdb->prefix}dab_state ORDER BY id LIMIT 1");

    return !!$stored;
  }

  static function total_work_available() {
    global $wpdb;
    $count = $wpdb->get_var("SELECT SUM(sites_left) from {$wpdb->prefix}dab_state");
    return $count;
  }

  private function __construct( $post_id = NULL ) {
    global $wpdb;

    $this->multi_handle = curl_multi_init();
    $this->state = array();
    $this->run_on_next_tick = array();
    $this->running = 0;
    $this->num_future_requests = 0;
    $this->post_id = $post_id;
  }

  function serialize() {
    return serialize( $this->run_on_next_tick );
  }

  function unserialize( $data ) {
    $this->multi_handle = curl_multi_init();
    $this->state = array();
    $this->running = 0;
    $this->run_on_next_tick = unserialize( $data );
  }

  function store() {
    global $wpdb;

    if ( $this->is_done() && isset( $this->id ) ) {
      $sql = "DELETE FROM {$wpdb->prefix}dab_state WHERE id = {$this->id}";
      $wpdb->query( $sql );
    }
    elseif (!$this->is_done() ) {
      if( isset( $this->id ) ) {
        # More work to do in this chunk.
          $wpdb->update(
          "{$wpdb->prefix}dab_state",
          array(
            'state' => serialize( $this ),
            'sites_left' => count( $this->run_on_next_tick )
          ),
          array( 'id' => $this->id ),
          array( "%s", "%d" )
        );
      }
      else {
        $res = $wpdb->insert(
          "{$wpdb->prefix}dab_state",
          array(
            'state' => serialize($this),
            'sites_left' => count( $this->run_on_next_tick ),
            'sites_total' => count( $this->run_on_next_tick ),
            'post_id' => $this->post_id
          ),
          array( "%s", "%d", "%d", "%d" )
        );
        $this->id = $wpdb->insert_id;
      }
    }
  }

  function _enqueue( $cb, $args = array() ) {
    $this->run_on_next_tick[] = array( $cb, $args );
  }

  protected function _add( $handle, $success = null, $failure = null, $args = NULL ) {
    $this->running = 1;

    curl_setopt( $handle, CURLOPT_HEADERFUNCTION, array( $this, 'on_header' ) );

    curl_multi_add_handle( $this->multi_handle, $handle );
    $this->state[ $handle ] = array(
      'success' => $success,
      'failure' => $failure,
      'args' => $args,
      'header' => ""
    );
  }

  function on_header( $handle, $header ) {
    $this->state[ $handle ][ 'header' ] .= $header;
    return strlen( $header );
  }

  function http_parse_headers( $header )
  {
    $ret = array();
    $fields = explode("\r\n", preg_replace('/\x0D\x0A[\x09\x20]+/', ' ', $header));
    foreach( $fields as $field ) {
      if( preg_match('/([^:]+):\s+(.+)/m', $field, $match) ) {
        $match[1] = preg_replace('/(?<=^|[\x09\x20\x2D])./e', 'strtoupper("\0")', strtolower(trim($match[1])));
        $h_name = strtolower( $match[1] );
        $h_val = trim( $match[2] );
        if( isset( $ret[ $h_name ] ) ) {
          $ret[ $h_name ][] = $h_val;
        }
        else {
          $ret[ $h_name ] = array( $h_val );
        }
      }
    }
    return $ret;
  }

  function extract_cookies( $str ) {
    $headers = $this->http_parse_headers( $str );
    if ( !isset( $headers['set-cookie'] ) ) {
      return FALSE;
    }

    $set_cookies = $headers['set-cookie'];

    $cookies = array();
    foreach (  $set_cookies as $line ) {
      $csplit = ( strpos( $line, ';' ) !== false ) ? explode( ';', $line ) : array( $line );
      $cdata = array();
      foreach( $csplit as $data ) {
        $cinfo = explode( '=', $data );
        $cinfo[0] = trim( $cinfo[0] );
        if( $cinfo[0] == 'expires' ) $cinfo[1] = strtotime( $cinfo[1] );
        if( $cinfo[0] == 'secure' ) $cinfo[1] = true;
        if( in_array( $cinfo[0], array( 'domain', 'expires', 'path', 'secure', 'comment' ) ) ) {
          $cdata[trim( $cinfo[0] )] = $cinfo[1];
        }
        else {
          $cdata['value']['key'] = $cinfo[0];
          $cdata['value']['value'] = $cinfo[1];
        }
      }
      $cookies[ $cdata['value']['key'] ] = $cdata['value']['key'].'='.$cdata['value']['value'];
    }
    return $cookies;
  }

  function is_done() {
    return !$this->running && count( $this->run_on_next_tick ) == 0;
  }

  function exec() {
    # Store and create a new array incase any ticks instantly re-queue themselves
    $ticks = $this->run_on_next_tick;
    $this->run_on_next_tick = array();
    foreach( $ticks as $run ) {
      dab_debug("Running next tick fn ".get_class($run[0][0]). "#{$run[0][1]}");
      call_user_func_array( $run[0], $run[1] );
    }


    do {
      # Give it some time to process something
      usleep( 10000 );
      curl_multi_exec( $this->multi_handle, $this->running );

      while ( $done = curl_multi_info_read( $this->multi_handle ) ) {
        $handle = $done[ 'handle' ];
        $info = curl_getinfo( $handle );
        $output = curl_multi_getcontent( $handle );

        $state = $this->state[ $handle ];
        $args = $state[ 'args' ];
        if ( $info[ 'http_code' ] == 200 ) {
          $callback = $state[ 'success' ];
        } else {
          $callback = $state[ 'failure' ];
        }

        $info[ 'cookies' ] = $this->extract_cookies( $state[ 'header' ] );

        # Remove the handle and the callbacks
        unset( $this->state[ $handle ] );

        curl_multi_remove_handle( $this->multi_handle, $handle );
        curl_close( $handle );

        if ( $callback ) {
          if ( $args ) {
            $args = array_merge( array($info, $output), $args );
          }
          else {
            $args = array( $info, $output );
          }

          $this->_enqueue( $callback, $args );
        }
      }
    } while ( $this->running > 0 );
  }

}
